@@ -61,29 +61,42 @@
 
 #define READ_CHUNK_SIZE 8192
 
+#define MAX_SYMLINKS_FOLLOWED 32
+
+
+/**
+ * gnome_vfs_format_file_size_for_display:
+ * @size:
+ * 
+ * Formats the file size passed in @bytes in a way that is easy for
+ * the user to read. Gives the size in bytes, kilobytes, megabytes or
+ * gigabytes, choosing whatever is appropriate.
+ * 
+ * Returns: a newly allocated string with the size ready to be shown.
+ **/
 
 gchar*
-gnome_vfs_format_file_size_for_display (GnomeVFSFileSize bytes)
+gnome_vfs_format_file_size_for_display (GnomeVFSFileSize size)
 {
-	if (bytes < (GnomeVFSFileSize) KILOBYTE_FACTOR) {
-		if (bytes == 1)
+	if (size < (GnomeVFSFileSize) KILOBYTE_FACTOR) {
+		if (size == 1)
 			return g_strdup (_("1 byte"));
 		else
 			return g_strdup_printf (_("%u bytes"),
-						       (guint) bytes);
+						       (guint) size);
 	} else {
 		gdouble displayed_size;
 
-		if (bytes < (GnomeVFSFileSize) MEGABYTE_FACTOR) {
-			displayed_size = (gdouble) bytes / KILOBYTE_FACTOR;
+		if (size < (GnomeVFSFileSize) MEGABYTE_FACTOR) {
+			displayed_size = (gdouble) size / KILOBYTE_FACTOR;
 			return g_strdup_printf (_("%.1f K"),
 						       displayed_size);
-		} else if (bytes < (GnomeVFSFileSize) GIGABYTE_FACTOR) {
-			displayed_size = (gdouble) bytes / MEGABYTE_FACTOR;
+		} else if (size < (GnomeVFSFileSize) GIGABYTE_FACTOR) {
+			displayed_size = (gdouble) size / MEGABYTE_FACTOR;
 			return g_strdup_printf (_("%.1f MB"),
 						       displayed_size);
 		} else {
-			displayed_size = (gdouble) bytes / GIGABYTE_FACTOR;
+			displayed_size = (gdouble) size / GIGABYTE_FACTOR;
 			return g_strdup_printf (_("%.1f GB"),
 						       displayed_size);
 		}
@@ -637,13 +650,12 @@ gnome_vfs_make_uri_canonical_old (const char *original_uri_text)
 	return result;
 }
 
-
 /**
- * gnome_vfs_make_canonical_pathname:
+ * gnome_vfs_make_path_name_canonical:
  * @path: a file path, relative or absolute
  * 
- * Calls gnome_vfs_canonicalize_pathname, allocating storage for the result and
- * providing for a cleaner memory management.
+ * Calls _gnome_vfs_canonicalize_pathname, allocating storage for the 
+ * result and providing for a cleaner memory management.
  * 
  * Return value: a canonical version of @path
  **/
@@ -654,7 +666,7 @@ gnome_vfs_make_path_name_canonical (const gchar *path)
 	char *result;
 
 	path_clone = g_strdup (path);
-	result = gnome_vfs_canonicalize_pathname (path_clone);
+	result = _gnome_vfs_canonicalize_pathname (path_clone);
 	if (result != path_clone) {
 		g_free (path_clone);
 		return g_strdup (result);
@@ -699,7 +711,7 @@ gnome_vfs_get_local_path_from_uri (const char *uri)
 {
 	const char *path_part;
 
-	if (!gnome_vfs_istr_has_prefix (uri, "file:/")) {
+	if (!_gnome_vfs_istr_has_prefix (uri, "file:/")) {
 		return NULL;
 	}
 	
@@ -708,9 +720,9 @@ gnome_vfs_get_local_path_from_uri (const char *uri)
 		return NULL;
 	}
 	
-	if (gnome_vfs_istr_has_prefix (path_part, "///")) {
+	if (_gnome_vfs_istr_has_prefix (path_part, "///")) {
 		path_part += 2;
-	} else if (gnome_vfs_istr_has_prefix (path_part, "//")) {
+	} else if (_gnome_vfs_istr_has_prefix (path_part, "//")) {
 		return NULL;
 	}
 
@@ -743,14 +755,19 @@ gnome_vfs_get_uri_from_local_path (const char *local_full_path)
 	return result;
 }
 
-/* gnome_vfs_get_volume_free_space
+/**
+ * gnome_vfs_get_volume_free_space:
+ * @vfs_uri:
+ * @size:
  * 
- * Return total amount of free space on a volume.
- * This only works for local file systems with
- * the file: scheme.
+ * Stores in @size the amount of free space on a volume.
+ * This only works for local file systems with the file: scheme.
+ *
+ * Returns: GNOME_VFS_OK on success, otherwise an error code
  */
 GnomeVFSResult
-gnome_vfs_get_volume_free_space (const GnomeVFSURI *vfs_uri, GnomeVFSFileSize *size)
+gnome_vfs_get_volume_free_space (const GnomeVFSURI *vfs_uri, 
+				 GnomeVFSFileSize  *size)
 {	
 	GnomeVFSFileSize free_blocks, block_size;
        	int statfs_result;
@@ -777,7 +794,7 @@ gnome_vfs_get_volume_free_space (const GnomeVFSURI *vfs_uri, GnomeVFSFileSize *s
 	scheme = gnome_vfs_uri_get_scheme (vfs_uri);
 	
         /* We only handle the file scheme for now */
-	if (g_ascii_strcasecmp (scheme, "file") != 0 || !gnome_vfs_istr_has_prefix (path, "/")) {
+	if (g_ascii_strcasecmp (scheme, "file") != 0 || !_gnome_vfs_istr_has_prefix (path, "/")) {
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 	}
 
@@ -824,7 +841,6 @@ hack_file_exists (const char *filename)
 	return stat (filename, &s) == 0;
 }
 
-
 char *
 gnome_vfs_icon_path_from_filename (const char *relative_filename)
 {
@@ -858,7 +874,6 @@ gnome_vfs_icon_path_from_filename (const char *relative_filename)
 	return NULL;
 }
 
-
 static char *
 strdup_to (const char *string, const char *end)
 {
@@ -891,7 +906,6 @@ is_executable_file (const char *path)
 	return TRUE;
 }
 
-
 static gboolean
 executable_in_path (const char *executable_name)
 {
@@ -939,8 +953,15 @@ get_executable_name_from_command_string (const char *command_string)
 	return g_strstrip (strdup_to (command_string, strchr (command_string, ' ')));
 }
 
-/* Returns TRUE if commmand_string starts with the full path for an executable
- * file, or starts with a command for an executable in $PATH.
+/**
+ * gnome_vfs_is_executable_command_string:
+ * @command_string:
+ * 
+ * Checks if @command_string starts with the full path of an executable file
+ * or an executable in $PATH.
+ *
+ * Returns: TRUE if command_string started with and executable file, 
+ * FALSE otherwise.
  */
 gboolean
 gnome_vfs_is_executable_command_string (const char *command_string)
@@ -984,16 +1005,17 @@ gnome_vfs_is_executable_command_string (const char *command_string)
 /**
  * gnome_vfs_read_entire_file:
  * @uri: URI of the file to read
- * @file_size: after reading the file, contains the size of the 
- * file read
- * @file_contents: contains the file_size bytes, the contents 
- * of the file at uri.
+ * @file_size: after reading the file, contains the size of the file read
+ * @file_contents: contains the file_size bytes, the contents of the file at @uri.
  * 
  * Reads an entire file into memory for convenience. Beware accidentally
  * loading large files into memory with this function.
  *
  * Return value: An integer representing the result of the operation
+ *
+ * Since: 2.2
  */
+
 GnomeVFSResult
 gnome_vfs_read_entire_file (const char *uri,
 			    int *file_size,
@@ -1132,7 +1154,7 @@ gnome_vfs_format_uri_for_display_internal (const char *uri, gboolean filenames_a
  *
  * Filter, modify, unescape and change URIs to make them appropriate
  * to display to users. The conversion is done such that the roundtrip
- * to UTf8 is reversible.
+ * to UTF-8 is reversible.
  * 
  * Rules:
  * 	file: URI's without fragments should appear as local paths
@@ -1141,8 +1163,11 @@ gnome_vfs_format_uri_for_display_internal (const char *uri, gboolean filenames_a
  *
  * @uri: a URI
  *
- * returns a g_malloc'd UTF8 string
+ * Returns: a newly allocated UTF-8 string
+ *
+ * Since: 2.2
  **/
+
 char *
 gnome_vfs_format_uri_for_display (const char *uri) 
 {
@@ -1301,22 +1326,19 @@ gnome_vfs_make_uri_from_input_internal (const char *text,
 	
 }
 
-
-
 /**
  * gnome_vfs_make_uri_from_input:
- *
- * Takes a user input path/URI and makes a valid URI
- * out of it
- *
  * @location: a possibly mangled "uri", in UTF8
  *
- * returns a newly allocated uri.
+ * Takes a user input path/URI and makes a valid URI out of it.
  *
  * This function is the reverse of gnome_vfs_format_uri_for_display
  * but it also handles the fact that the user could have typed
  * arbitrary UTF8 in the entry showing the string.
  *
+ * Returns: a newly allocated uri.
+ *
+ * Since: 2.2
  **/
 char *
 gnome_vfs_make_uri_from_input (const char *location)
@@ -1328,6 +1350,18 @@ gnome_vfs_make_uri_from_input (const char *location)
 	return gnome_vfs_make_uri_from_input_internal (location, broken_filenames, TRUE);
 }
 
+/**
+ * gnome_vfs_make_uri_canonical_strip_fragment:
+ * @uri:
+ *
+ * If the @uri passed contains a fragment (anything after a '#') strips if,
+ * then makes the URI canonical.
+ *
+ * Returns: a newly allocated string containing a canonical URI.
+ *
+ * Since: 2.2
+ **/
+
 char *
 gnome_vfs_make_uri_canonical_strip_fragment (const char *uri)
 {
@@ -1345,7 +1379,6 @@ gnome_vfs_make_uri_canonical_strip_fragment (const char *uri)
 	return canonical;
 }
 
-
 static gboolean
 uris_match (const char *uri_1, const char *uri_2, gboolean ignore_fragments)
 {
@@ -1368,6 +1401,18 @@ uris_match (const char *uri_1, const char *uri_2, gboolean ignore_fragments)
 	return result;
 }
 
+/**
+ * gnome_vfs_uris_match:
+ * @uri_1: stringified URI to compare with @uri_2.
+ * @uri_2: stringified URI to compare with @uri_1.
+ * 
+ * Compare two URIs.
+ *
+ * Return value: TRUE if they are the same, FALSE otherwise.
+ *
+ * Since: 2.2
+ **/
+
 gboolean
 gnome_vfs_uris_match (const char *uri_1, const char *uri_2)
 {
@@ -1407,7 +1452,7 @@ gnome_vfs_uri_is_local_scheme (const char *uri)
 
 	is_local_scheme = FALSE;
 	for (temp_scheme = *local_schemes, i = 0; temp_scheme != NULL; i++, temp_scheme = local_schemes[i]) {
-		is_local_scheme = gnome_vfs_istr_has_prefix (uri, temp_scheme);
+		is_local_scheme = _gnome_vfs_istr_has_prefix (uri, temp_scheme);
 		if (is_local_scheme) {
 			break;
 		}
@@ -1482,7 +1527,18 @@ gnome_vfs_handle_trailing_slashes (const char *uri)
 	return uri_copy;
 }
 
-
+/**
+ * gnome_vfs_make_uri_canonical:
+ * @uri: and absolute or relative URI, it might have scheme.
+ *
+ * Standarizes the format of the uri being passed, so that it can be used
+ * later in other functions that expect a canonical URI.
+ *
+ * Returns: a newly allocated string that contains the canonical 
+ * representation of @uri.
+ *
+ * Since: 2.2
+ **/
 
 char *
 gnome_vfs_make_uri_canonical (const char *uri)
@@ -1569,6 +1625,17 @@ gnome_vfs_make_uri_canonical (const char *uri)
 	return canonical_uri;
 }
 
+/**
+ * gnome_vfs_get_uri_scheme:
+ * @uri: a stringified URI
+ *
+ * Retrieve the scheme used in @uri 
+ *
+ * Return value: A string containing the scheme
+ *
+ * Since: 2.2
+ **/
+
 char *
 gnome_vfs_get_uri_scheme (const char *uri)
 {
@@ -1615,6 +1682,7 @@ file_uri_from_local_relative_path (const char *location)
 
 /**
  * gnome_vfs_make_uri_from_shell_arg:
+ * @location: a possibly mangled "uri"
  *
  * Similar to gnome_vfs_make_uri_from_input, except that:
  * 
@@ -1622,11 +1690,11 @@ file_uri_from_local_relative_path (const char *location)
  * 2) doesn't bother stripping leading/trailing white space
  * 3) doesn't bother with ~ expansion--that's done by the shell
  *
- * @location: a possibly mangled "uri"
- *
- * returns a newly allocated uri
+ * Returns: a newly allocated uri
  *
+ * Since: 2.2
  **/
+
 char *
 gnome_vfs_make_uri_from_shell_arg (const char *location)
 {
@@ -1748,8 +1816,6 @@ remove_internal_relative_components (char *uri_current)
 	}	
 }
 
-
-
 /**
  * gnome_vfs_make_uri_full_from_relative:
  * 
@@ -1758,10 +1824,7 @@ remove_internal_relative_components (char *uri_current)
  *
  * Return value: the URI (NULL for some bad errors).
  *
- * FIXME: This code has been copied from gnome_vfs-mozilla-content-view
- * because gnome_vfs-mozilla-content-view cannot link with libgnome_vfs-extensions
- * due to lame license issues.  Really, this belongs in gnome-vfs, but was added
- * after the Gnome 1.4 gnome-vfs API freeze
+ * Since: 2.2
  **/
 
 char *
@@ -1891,3 +1954,166 @@ gnome_vfs_make_uri_full_from_relative (const char *base_uri, const char *relativ
 	return result;
 }
 
+GnomeVFSResult
+_gnome_vfs_uri_resolve_all_symlinks_uri (GnomeVFSURI *uri,
+					 GnomeVFSURI **result_uri)
+{
+	GnomeVFSURI *new_uri, *resolved_uri;
+	GnomeVFSFileInfo *info;
+	GnomeVFSResult res;
+	char *p;
+	int n_followed_symlinks;
+
+	/* Ref the original uri so we don't lose it */
+	uri = gnome_vfs_uri_ref (uri);
+
+	*result_uri = NULL;
+
+	info = gnome_vfs_file_info_new ();
+
+	p = uri->text;
+	n_followed_symlinks = 0;
+	while (*p != 0) {
+		while (*p == GNOME_VFS_URI_PATH_CHR)
+			p++;
+		while (*p != 0 && *p != GNOME_VFS_URI_PATH_CHR)
+			p++;
+
+		new_uri = gnome_vfs_uri_dup (uri);
+		g_free (new_uri->text);
+		new_uri->text = g_strndup (uri->text, p - uri->text);
+		
+		gnome_vfs_file_info_clear (info);
+		res = gnome_vfs_get_file_info_uri (new_uri, info, GNOME_VFS_FILE_INFO_DEFAULT);
+		if (res != GNOME_VFS_OK) {
+			gnome_vfs_uri_unref (new_uri);
+			goto out;
+		}
+		if (info->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK &&
+		    info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME) {
+			n_followed_symlinks++;
+			if (n_followed_symlinks > MAX_SYMLINKS_FOLLOWED) {
+				res = GNOME_VFS_ERROR_TOO_MANY_LINKS;
+				gnome_vfs_uri_unref (new_uri);
+				goto out;
+			}
+			resolved_uri = gnome_vfs_uri_resolve_relative (new_uri,
+								       info->symlink_name);
+			if (*p != 0) {
+				gnome_vfs_uri_unref (uri);
+				uri = gnome_vfs_uri_append_path (resolved_uri, p);
+				gnome_vfs_uri_unref (resolved_uri);
+			} else {
+				gnome_vfs_uri_unref (uri);
+				uri = resolved_uri;
+			}
+
+			p = uri->text;
+		} 
+		gnome_vfs_uri_unref (new_uri);
+	}
+
+	res = GNOME_VFS_OK;
+	*result_uri = gnome_vfs_uri_dup (uri);
+ out:
+	gnome_vfs_file_info_unref (info);
+	gnome_vfs_uri_unref (uri);
+	return res;
+}
+
+GnomeVFSResult
+_gnome_vfs_uri_resolve_all_symlinks (const char *text_uri,
+				     char **resolved_text_uri)
+{
+	GnomeVFSURI *uri, *resolved_uri;
+	GnomeVFSResult res;
+
+	*resolved_text_uri = NULL;
+
+	uri = gnome_vfs_uri_new (text_uri);
+	if (uri == NULL || uri->text == NULL) {
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
+	}
+
+	res = _gnome_vfs_uri_resolve_all_symlinks_uri (uri, &resolved_uri);
+
+	if (res == GNOME_VFS_OK) {
+		*resolved_text_uri = gnome_vfs_uri_to_string (resolved_uri, GNOME_VFS_URI_HIDE_NONE);
+		gnome_vfs_uri_unref (resolved_uri);
+	}
+	return res;
+}
+
+gboolean 
+_gnome_vfs_uri_is_in_subdir (GnomeVFSURI *uri, GnomeVFSURI *dir)
+{
+	GnomeVFSFileInfo *dirinfo, *info;
+	GnomeVFSURI *resolved_dir, *parent, *tmp;
+	GnomeVFSResult res;
+	gboolean is_in_dir;
+
+	resolved_dir = NULL;
+	parent = NULL;
+
+	is_in_dir = FALSE;
+	
+	dirinfo = gnome_vfs_file_info_new ();
+	info = gnome_vfs_file_info_new ();
+
+	res = gnome_vfs_get_file_info_uri (dir, dirinfo, GNOME_VFS_FILE_INFO_DEFAULT);
+	if (res != GNOME_VFS_OK || dirinfo->type != GNOME_VFS_FILE_TYPE_DIRECTORY) {
+		goto out;
+	}
+
+	res = _gnome_vfs_uri_resolve_all_symlinks_uri (dir, &resolved_dir);
+	if (res != GNOME_VFS_OK) {
+		goto out;
+	}
+	
+	res = _gnome_vfs_uri_resolve_all_symlinks_uri (uri, &tmp);
+	if (res != GNOME_VFS_OK) {
+		goto out;
+	}
+	
+	parent = gnome_vfs_uri_get_parent (tmp);
+	gnome_vfs_uri_unref (tmp);
+
+	while (parent != NULL) {
+		res = gnome_vfs_get_file_info_uri (parent, info, GNOME_VFS_FILE_INFO_DEFAULT);
+		if (res != GNOME_VFS_OK) {
+			break;
+		}
+
+		if (dirinfo->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_DEVICE &&
+		    dirinfo->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_INODE &&
+		    info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_DEVICE &&
+		    info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_INODE) {
+			if (dirinfo->device == info->device &&
+			    dirinfo->inode == info->inode) {
+				is_in_dir = TRUE;
+				break;
+			}
+		} else {
+			if (gnome_vfs_uri_equal (dir, parent)) {
+				is_in_dir = TRUE;
+				break;
+			}
+		}
+		
+		tmp = gnome_vfs_uri_get_parent (parent);
+		gnome_vfs_uri_unref (parent);
+		parent = tmp;
+	}
+
+ out:
+	if (resolved_dir != NULL) {
+		gnome_vfs_uri_unref (resolved_dir);
+	}
+	if (parent != NULL) {
+		gnome_vfs_uri_unref (parent);
+	}
+	gnome_vfs_file_info_unref (info);
+	return is_in_dir;
+}
+
+			  
